// -*- C++ -*-
/*!
 * @file  Houghlines.cpp
 * @brief Draw Houghlines
 * $Date$
 *
 * $Id$
 */

#include "Houghlines.h"
#include "Camera.h"

IplImage* src = cvCreateImage( cvSize(160,120),IPL_DEPTH_8U, 1);
IplImage* osrc = cvCreateImage( cvGetSize(src),IPL_DEPTH_8U, 3);//3);//NULL;
IplImage* dst = cvCreateImage( cvGetSize(src), 8, 1 );
IplImage* color_dst = cvCreateImage( cvGetSize(src), 8, 3 );

// Module specification
// <rtc-template block="module_spec">
static const char* houghlines_spec[] =
  {
    "implementation_id", "Houghlines",
    "type_name",         "Houghlines",
    "description",       "Draw Houghlines",
    "version",           "0.1",
    "vendor",            "Osaka Univ.",
    "category",          "Generic",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.int_param", "0",

    ""
  };
// </rtc-template>

Houghlines::Houghlines(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_InputImageIn("InputImage", m_InputImage),
    m_OutputImageOut("OutputImage", m_OutputImage),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("InputImage", m_InputImageIn);
  
  // Set OutPort buffer
  registerOutPort("OutputImage", m_OutputImageOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

Houghlines::~Houghlines()
{
}


RTC::ReturnCode_t Houghlines::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("int_param", m_int_param, "0");
  //printf("initialize\n");
  //cvNamedWindow("Source", 0);
  //cvNamedWindow("Processed", 0);
  // </rtc-template>
  return RTC::RTC_OK;
}




RTC::ReturnCode_t Houghlines::onFinalize()
{
  //cvDestroyWindow("Source");
  //cvDestroyWindow("Processed");

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t Houghlines::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Houghlines::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Houghlines::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Houghlines::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t Houghlines::onExecute(RTC::UniqueId ec_id)
{
  CvMemStorage* storage = cvCreateMemStorage(0);
  CvSeq* lines = 0;
  unsigned int i=0;

  //if(!src)
  //  return RTC::RTC_OK;
  //printf("1\n");

  /*inport part begin*/
  m_InputImageIn.read();

  for(i=0;i<m_InputImage.data.length();i++){
    osrc->imageData[i]=m_InputImage.data[i];
  }
  /*inport part end*/

  /*main process begin*/
  cvCvtColor( osrc, src, CV_BGR2GRAY );
  cvCanny( src, dst, 50, 200, 3 );
  cvCvtColor( dst, color_dst, CV_GRAY2BGR );
  //copy src to color_dst
#if 0
  lines = cvHoughLines2( dst, storage, CV_HOUGH_STANDARD, 1, CV_PI/180, 100, 0, 0 );

  for( i = 0; i < MIN(lines->total,100); i++ )  {
    float* line = (float*)cvGetSeqElem(lines,i);
    float rho = line[0];
    float theta = line[1];
    CvPoint pt1, pt2;
    double a = cos(theta), b = sin(theta);
    double x0 = a*rho, y0 = b*rho;
    pt1.x = cvRound(x0 + 1000*(-b));
    pt1.y = cvRound(y0 + 1000*(a));
    pt2.x = cvRound(x0 - 1000*(-b));
    pt2.y = cvRound(y0 - 1000*(a));
    //cvLine( color_dst, pt1, pt2, CV_RGB(255,0,0), 3, CV_AA, 0 );
    cvLine( osrc, pt1, pt2, CV_RGB(255,0,0), m_int_param, CV_AA, 0 );
  }
#else
  lines = cvHoughLines2( dst, storage, CV_HOUGH_PROBABILISTIC, 1, CV_PI/180, 50, 50, 10 );
  for( int i = 0; i < lines->total; i++ )  {
    CvPoint* line = (CvPoint*)cvGetSeqElem(lines,i);
    cvLine( osrc , line[0], line[1], CV_RGB(255,0,0), m_int_param, CV_AA, 0 );
    //cvLine( color_dst, line[0], line[1], CV_RGB(255,0,0), 3, CV_AA, 0 );
  }
#endif
  /*main process end*/

  /*outport part begin*/
  //cvShowImage("Source", src);
  //cvShowImage("Processed", color_dst);
  //m_OutputImage.data.length(color_dst->imageSize);
  m_OutputImage.data.length(osrc->imageSize);

  for(int i=0;i<osrc->imageSize;i++){
    //for(int i=0;i<color_dst->imageSize;i++){
    //m_OutputImage.data[i]=color_dst->imageData[i];
    m_OutputImage.data[i]=osrc->imageData[i];
  }

  m_OutputImageOut.write();
  /*outport part end*/

  cvWaitKey(100);

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t Houghlines::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Houghlines::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Houghlines::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Houghlines::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Houghlines::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void HoughlinesInit(RTC::Manager* manager)
  {
    RTC::Properties profile(houghlines_spec);
    manager->registerFactory(profile,
                             RTC::Create<Houghlines>,
                             RTC::Delete<Houghlines>);
  }
  
};


