// -*- C++ -*-
/*!
 * @file  CaribComponent.cpp
 * @brief Carib Component
 * $Date$
 *
 * $Id$
 */

#include "CaribComponent.h"
#include "Camera.h"

int numCols,numLines,imageIndex=0;
CvPoint2D32f *imageCorners, *corners;
int m_width,m_height;
int d_count=0;
CvPoint tempP;

IplImage* src =NULL;// = cvCreateImage( cvSize(320,240),IPL_DEPTH_8U, 1);
IplImage* osrc=NULL;// = cvCreateImage( cvGetSize(src),IPL_DEPTH_8U, 3);//3);//NULL;
IplImage* odst =NULL;// cvCreateImage( cvGetSize(src), 8, 1 );
IplImage* dst =NULL;// cvCreateImage( cvGetSize(src), 8, 1 );
IplImage* cdst =NULL;// cvCreateImage( cvGetSize(src), 8, 3 );

CvPoint textPos;
CvFont textFont;
CvScalar color;
char *stateStr = new char[128];

int phase=0;
int m_calib=0;

// Module specification
// <rtc-template block="module_spec">
static const char* caribcomponent_spec[] =
  {
    "implementation_id", "CaribComponent",
    "type_name",         "CaribComponent",
    "description",       "Carib Component",
    "version",           "0.1",
    "vendor",            "OsakaUniv",
    "category",          "Generic",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.int_width", "320",
    "conf.default.int_height", "240",
    "conf.default.int_param1", "9",
    "conf.default.int_param2", "9",
    "conf.default.int_param3", "5",

    ""
  };
// </rtc-template>

CaribComponent::CaribComponent(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_InImageDataIn("InImageData", m_InImageData),
    m_OutImageDataOut("OutImageData", m_OutImageData),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("InImageData", m_InImageDataIn);
  
  // Set OutPort buffer
  registerOutPort("OutImageData", m_OutImageDataOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

CaribComponent::~CaribComponent()
{
}




RTC::ReturnCode_t CaribComponent::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("int_width", m_int_width, "320");
  bindParameter("int_height", m_int_height, "240");
  bindParameter("int_param1", m_int_param1, "9");
  bindParameter("int_param2", m_int_param2, "9");
  bindParameter("int_param3", m_int_param3, "5");
  
  cvInitFont(&textFont,CV_FONT_VECTOR0, 0.4f, 0.7f, 0.0f,2);
  color.val[0]=0; color.val[1]=0; color.val[2]=0xff; color.val[3]=0;

  imageCorners = new CvPoint2D32f[(m_int_param1-1)*(m_int_param2-1)];
  corners = new CvPoint2D32f[(m_int_param1-1)*(m_int_param2-1)*m_int_param3];

  // m_width=m_int_width;
  //m_height=m_int_height;

  // </rtc-template>
  return RTC::RTC_OK;
}



/*
RTC::ReturnCode_t CaribComponent::onFinalize()
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t CaribComponent::onStartup(RTC::UniqueId ec_id)
{
  m_calib=1;
  // This  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t CaribComponent::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t CaribComponent::onActivated(RTC::UniqueId ec_id)
{
  src = cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 1);
  osrc = cvCreateImage( cvGetSize(src),IPL_DEPTH_8U, 3);//3);//NULL;

  odst = cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 1);

  dst = cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 1);

  numCols=m_int_param1;
  numLines=m_int_param2;
  m_width=m_int_width;
  m_height=m_int_height;
  printf("m_width %d \n",m_width);
  printf("m_height %d \n",m_height);

  return RTC::RTC_OK;
}



RTC::ReturnCode_t CaribComponent::onDeactivated(RTC::UniqueId ec_id)
{
  cvReleaseImage(&osrc);
  cvReleaseImage(&src);
  cvReleaseImage(&odst);
  cvReleaseImage(&dst);

  return RTC::RTC_OK;
}



bool
getCorners (int camId)
{

  int corners_found = (numCols - 1) * (numLines - 1);

  //cvCopyImage(osrc,caliIm);

  cvCvtColor (osrc, src, CV_BGR2GRAY);

  IplImage *threshold =
    cvCreateImage (cvSize (m_width, m_height), IPL_DEPTH_8U, 1);

  CvMemStorage *memstore = cvCreateMemStorage ();

  int returnVal = cvFindChessBoardCornerGuesses (src, threshold, memstore,
						 cvSize (numCols - 1,
							 numLines - 1),
						 imageCorners,
						 &corners_found);
  //кѤߥʡɽ
  if(returnVal == 0)
  {//쥯Ȳ
	  CvPoint point;
	  int count = 0;

	  do{
		  point.x = (int)(imageCorners + count)->x;
		  point.y = (int)(imageCorners + count)->y;
		  cvCircle (osrc, point, 10, CV_RGB (255, 0, 0), 2);
		  count++;
	  }while(count < corners_found);
	  printf("Image is wrong(%d points are found)\n",corners_found);
	  return false;
  }
  else
    {
      printf("Image is correct %d\n",corners_found);
    }
  cvFindCornerSubPix (src,
		      imageCorners, corners_found,
		      cvSize (5, 5), cvSize (-1, -1),
		      cvTermCriteria (CV_TERMCRIT_ITER, 100, 0.1));

  cvReleaseMemStorage (&memstore);
  cvReleaseImage (&threshold);

  int numCornersPerImage=(numCols-1)*(numLines-1);

  if (corners_found != numCornersPerImage){
    printf("failed number %d \n",numCornersPerImage);
    return false;
  }
  
  for (int i = 0; i < numCornersPerImage; i++)
    {
      corners[i + imageIndex * numCornersPerImage].x = imageCorners[i].x;
      corners[i + imageIndex * numCornersPerImage].y = imageCorners[i].y;
      tempP.x = imageCorners[i].x;
      tempP.y = imageCorners[i].y;
      cvCircle (osrc, tempP, 10, CV_RGB (255, 0, 0), 2);
      cvCircle (osrc, tempP, 1, CV_RGB (255, 0, 0), CV_FILLED);
    }
  
  
  return true;
}



RTC::ReturnCode_t CaribComponent::onExecute(RTC::UniqueId ec_id)
{
  //int calib=1;

  textPos.x=3;
  textPos.y=20;
  
  /*inport part*/
  m_InImageDataIn.read();
  if(m_InImageData.data.length()==m_int_width*m_int_height*3){
    memcpy(osrc->imageData,(void *)&(m_InImageData.data[0]),m_int_width*m_int_height*3);
    
    
    
    if (m_calib&&imageIndex<m_int_param3)
      {
	if(getCorners(0) == false){
	  phase=1;
	}else{
	  m_calib=0;
	  phase=2;
	}
	d_count=0;
	//m_calib=0;
	imageIndex++;
      }
    if(d_count>200){
      phase=0;
      //calib=1;
    }
    
    if(imageIndex==m_int_param3){
      phase=3;
    }

    switch(phase)
      {
      case 0:
	stateStr="Press any Key";
	break;
      case 1:
	stateStr="Get Corners: FALSE";
	break;
      case 2:
	stateStr="Get Corners: OK";
	break;
      case 3:
	stateStr="Calc Calib mat.: OK";
	break;
      case 4:
	stateStr="test 4: OK";
	break;
      }
    d_count++;
    
    cvPutText(osrc,stateStr,textPos,&textFont,color);
    
    
    /*outport part begin*/
    m_OutImageData.data.length(m_int_width*m_int_height*3);
    memcpy((void *)&(m_OutImageData.data[0]),osrc->imageData,m_int_width*m_int_height*3);
    m_OutImageDataOut.write();
    /*outport part end*/
    
  }
  
  cvWaitKey(100);

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t CaribComponent::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CaribComponent::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

 /*
RTC::ReturnCode_t CaribComponent::onReset(RTC::UniqueId ec_id)
{
  m_calib=1;
  return RTC::RTC_OK;
}
 */

/*
RTC::ReturnCode_t CaribComponent::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CaribComponent::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void CaribComponentInit(RTC::Manager* manager)
  {
    RTC::Properties profile(caribcomponent_spec);
    manager->registerFactory(profile,
                             RTC::Create<CaribComponent>,
                             RTC::Delete<CaribComponent>);
  }
  
};


