// -*- C++ -*-
/*!
 * @file  morphology.cpp
 * @brief morphology Component
 * $Date$
 *
 * $Id$
 */

#include "morphology.h"
#include "Camera.h"

IplImage* src = cvCreateImage( cvSize(320,240),IPL_DEPTH_8U, 3);
IplImage* dst = cvCreateImage( cvSize(320,240),IPL_DEPTH_8U, 3);
//IplImage* dst = 0;

IplConvKernel* element = 0;
int element_shape = CV_SHAPE_RECT;

//the address of variable which receives trackbar position update 
int max_iters = 10;
int open_close_pos = 0;
int erode_dilate_pos = 0;
int flag=0;

// Module specification
// <rtc-template block="module_spec">
static const char* morphology_spec[] =
  {
    "implementation_id", "morphology",
    "type_name",         "morphology",
    "description",       "morphology Component",
    "version",           "0.1",
    "vendor",            "OsakaUniv",
    "category",          "Generic",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables

    ""
  };
// </rtc-template>

morphology::morphology(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_ImageDataInIn("ImageDataIn", m_ImageDataIn),
    m_ImageDataOutOut("ImageDataOut", m_ImageDataOut),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("ImageDataIn", m_ImageDataInIn);
  
  // Set OutPort buffer
  registerOutPort("ImageDataOut", m_ImageDataOutOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

morphology::~morphology()
{
}

void OpenClose(int pos)   
{
    int n = open_close_pos - max_iters;
    int an = n > 0 ? n : -n;
    element = cvCreateStructuringElementEx( an*2+1, an*2+1, an, an, element_shape, 0 );
    if( n < 0 )
    {
        cvErode(src,dst,element,1);
        cvDilate(dst,dst,element,1);
    }
    else
    {
        cvDilate(src,dst,element,1);
        cvErode(dst,dst,element,1);
    }
    cvReleaseStructuringElement(&element);
    //    cvShowImage("Open/Close",dst);
}   

RTC::ReturnCode_t morphology::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  open_close_pos=2;
  // </rtc-template>
  return RTC::RTC_OK;
}




RTC::ReturnCode_t morphology::onFinalize()
{
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t morphology::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t morphology::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t morphology::onActivated(RTC::UniqueId ec_id)
{
  //int i;
    /*inport part begin*/
  /*
  m_ImageDataInIn.read();
  for(i=0;i< 320*240*3;i++){
    src->imageData[i]=m_ImageDataIn.data[i];
  }
  */

  //dst = cvCloneImage(src);


  return RTC::RTC_OK;
}



RTC::ReturnCode_t morphology::onDeactivated(RTC::UniqueId ec_id)
{
  flag=0;
  return RTC::RTC_OK;
}



RTC::ReturnCode_t morphology::onExecute(RTC::UniqueId ec_id)
{
  int i;
  
  /*inport part begin*/
  m_ImageDataInIn.read();
  if(m_ImageDataIn.data.length()==src->imageSize){
  memcpy(src->imageData,(void *)&(m_ImageDataIn.data[0]),320*240*3);//m_ImageDataIn.data.length());

  /*
  if(m_ImageDataIn.data.length()==320*240*3){
    if(flag==0){
      dst = cvCloneImage(src);
      flag=1;
    }
  }
*/

  dst = cvCloneImage(src);

  OpenClose(open_close_pos);
   
  m_ImageDataOut.data.length(dst->imageSize);
  
  for(i=0;i< dst->imageSize;i++){
    m_ImageDataOut.data[i]=dst->imageData[i];
  }
  m_ImageDataOutOut.write();
  }

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t morphology::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t morphology::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t morphology::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t morphology::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t morphology::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void morphologyInit(RTC::Manager* manager)
  {
    RTC::Properties profile(morphology_spec);
    manager->registerFactory(profile,
                             RTC::Create<morphology>,
                             RTC::Delete<morphology>);
  }
  
};





