// -*- C++ -*-
/*!
 * @file  RelayComponent.cpp
 * @brief Relay Component
 * $Date$
 *
 * $Id$
 */

#include "RelayComponent.h"
#include "Camera.h"

int start_flag=0;
int Wtime;

IplImage* osrc =NULL;

// Module specification
// <rtc-template block="module_spec">
static const char* relaycomponent_spec[] =
  {
    "implementation_id", "RelayComponent",
    "type_name",         "RelayComponent",
    "description",       "Relay Component",
    "version",           "0.1",
    "vendor",            "OsakaUniv",
    "category",          "Generic",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.int_width", "320",
    "conf.default.int_height", "240",
    "conf.default.int_param", "1",

    ""
  };
// </rtc-template>

RelayComponent::RelayComponent(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_InImageDataIn("InImageData", m_InImageData),
    m_OutImageDataOut("OutImageData", m_OutImageData),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("InImageData", m_InImageDataIn);
  
  // Set OutPort buffer
  registerOutPort("OutImageData", m_OutImageDataOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

RelayComponent::~RelayComponent()
{
}


RTC::ReturnCode_t RelayComponent::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("int_width", m_int_width, "320");
  bindParameter("int_height", m_int_height, "240");
  bindParameter("int_param", m_int_param, "1");
  
  Wtime=m_int_param;

  // </rtc-template>
  return RTC::RTC_OK;
}



/*
RTC::ReturnCode_t RelayComponent::onFinalize()
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t RelayComponent::onStartup(RTC::UniqueId ec_id)
{
  printf("test 2 \n");
  start_flag=1;

  return RTC::RTC_OK;
}



RTC::ReturnCode_t RelayComponent::onShutdown(RTC::UniqueId ec_id)
{

  start_flag=0;

  return RTC::RTC_OK;
}



RTC::ReturnCode_t RelayComponent::onActivated(RTC::UniqueId ec_id)
{
  osrc = cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 3);

  return RTC::RTC_OK;
}



RTC::ReturnCode_t RelayComponent::onDeactivated(RTC::UniqueId ec_id)
{
  cvReleaseImage(&osrc);
  return RTC::RTC_OK;
}



RTC::ReturnCode_t RelayComponent::onExecute(RTC::UniqueId ec_id)
{

    if(start_flag==1){
    /*inport part*/
      m_InImageDataIn.read();
      if(m_InImageData.data.length()==m_int_width*m_int_height*3){
	memcpy(osrc->imageData,(void *)&(m_InImageData.data[0]),m_int_width*m_int_height*3);
	Wtime=m_int_param;
	if(Wtime<1){
	  cvWaitKey(1);
	}else{
	  cvWaitKey(Wtime);
	}

	/*outport part begin*/
	m_OutImageData.data.length(m_int_width*m_int_height*3);
	memcpy((void *)&(m_OutImageData.data[0]),osrc->imageData,m_int_width*m_int_height*3);
	m_OutImageDataOut.write();
	/*outport part end*/
      }
    }

  cvWaitKey(100);

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t RelayComponent::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RelayComponent::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RelayComponent::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RelayComponent::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RelayComponent::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void RelayComponentInit(RTC::Manager* manager)
  {
    RTC::Properties profile(relaycomponent_spec);
    manager->registerFactory(profile,
                             RTC::Create<RelayComponent>,
                             RTC::Delete<RelayComponent>);
  }
  
};


