// -*- C++ -*-
/*!
 * @file  RotationComponent.cpp
 * @brief Rotation Component
 * $Date$
 *
 * $Id$
 */

#include "RotationComponent.h"
#include "Camera.h"

IplImage* src =NULL;// = cvCreateImage( cvSize(320,240),IPL_DEPTH_8U, 1);
IplImage* osrc=NULL;// = cvCreateImage( cvGetSize(src),IPL_DEPTH_8U, 3);//3);//NULL;
IplImage* odst =NULL;// cvCreateImage( cvGetSize(src), 8, 1 );
//IplImage* dst =NULL;// cvCreateImage( cvGetSize(src), 8, 1 );
//IplImage* cdst =NULL;// cvCreateImage( cvGetSize(src), 8, 3 );

// Module specification
// <rtc-template block="module_spec">
static const char* rotationcomponent_spec[] =
  {
    "implementation_id", "RotationComponent",
    "type_name",         "RotationComponent",
    "description",       "Rotation Component",
    "version",           "0.1",
    "vendor",            "OsakaUniv",
    "category",          "Generic",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.int_width", "320",
    "conf.default.int_height", "240",
    "conf.default.f_angle", "40",
    "conf.default.f_scale", "1",

    ""
  };
// </rtc-template>

RotationComponent::RotationComponent(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_InImageDataIn("InImageData", m_InImageData),
    m_OutImageDataOut("OutImageData", m_OutImageData),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("InImageData", m_InImageDataIn);
  
  // Set OutPort buffer
  registerOutPort("OutImageData", m_OutImageDataOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

RotationComponent::~RotationComponent()
{
}


RTC::ReturnCode_t RotationComponent::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("int_width", m_int_width, "320");
  bindParameter("int_height", m_int_height, "240");
  bindParameter("f_angle", m_f_angle, "40");
  bindParameter("f_scale", m_f_scale, "1");
  
  // </rtc-template>
  return RTC::RTC_OK;
}



/*
RTC::ReturnCode_t RotationComponent::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RotationComponent::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RotationComponent::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t RotationComponent::onActivated(RTC::UniqueId ec_id)
{
  int i;


  src = cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 1);
  osrc = cvCreateImage( cvGetSize(src),IPL_DEPTH_8U, 3);//3);//NULL;

  odst = cvCreateImage( cvGetSize(src),IPL_DEPTH_8U, 3);//3);//NULL;

  //dst= cvCreateImage( cvSize(m_int_width+1-cdst->width,m_int_height-cdst->height+1),IPL_DEPTH_32F, 1);

  /*inport part begin*/
  /*
  m_InImageDataIn.read();
  
  if(m_InImageData.data.length()>0){
    memcpy(osrc->imageData,(void *)&(m_InImageData.data[0]),m_int_width*m_int_height*3);//m_InImageData.data.length());
  }  
  
  for(i=0;i<m_InImageData.data.length();i++){
    osrc->imageData[i]=m_InImageData.data[i];
  }
  */

  /*inport part end*/


  return RTC::RTC_OK;
}



RTC::ReturnCode_t RotationComponent::onDeactivated(RTC::UniqueId ec_id)
{



  return RTC::RTC_OK;
}



RTC::ReturnCode_t RotationComponent::onExecute(RTC::UniqueId ec_id)
{
  /*inport part begin*/
  m_InImageDataIn.read();

  if(m_InImageData.data.length()==m_int_width*m_int_height*3){
    memcpy(osrc->imageData,(void *)&(m_InImageData.data[0]),m_int_width*m_int_height*3);//m_InImageData.data.length());
  
    CvPoint2D32f center =cvPoint2D32f(osrc->width/2.0,osrc->height/2.0);
    CvMat *rot_mat=cvCreateMat(2,3,CV_32FC1);
    //cv2DRotationMatrix(center,40.0,1.0,rot_mat);//center,m_f_angle,m_f_scale,rot_mat);
    cv2DRotationMatrix(center,m_f_angle,m_f_scale,rot_mat);

    cvWarpAffine(osrc,odst,rot_mat,CV_INTER_LINEAR+CV_WARP_FILL_OUTLIERS);
 
    
    /*outport part begin*/
    m_OutImageData.data.length(odst->imageSize);
    memcpy((void *)&(m_OutImageData.data[0]),odst->imageData,m_int_width*m_int_height*3);//m_InImageData.data.length());
    m_OutImageDataOut.write();
    /*outport part end*/
  }

    cvWaitKey(100);

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t RotationComponent::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RotationComponent::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RotationComponent::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RotationComponent::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RotationComponent::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void RotationComponentInit(RTC::Manager* manager)
  {
    RTC::Properties profile(rotationcomponent_spec);
    manager->registerFactory(profile,
                             RTC::Create<RotationComponent>,
                             RTC::Delete<RotationComponent>);
  }
  
};


